@@ -1,9 +1,26 @@
+/*
+ * Copyright (c) 2022 MiLaboratories Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package com.milaboratory.core.alignment;
 
 import com.milaboratory.core.Range;
 import com.milaboratory.core.mutations.MutationsBuilder;
 import com.milaboratory.core.sequence.NucleotideSequence;
 
+import static java.lang.Math.max;
+
 /**
  * @author Dmitry Bolotin
  * @author Stanislav Poslavsky
@@ -53,8 +70,8 @@ public class BandedAffineAligner {
         final int gapExtensionPenalty = scoring.getGapExtensionPenalty();
 
         for (i = 0; i < length1; ++i) {
-            to = Math.min(i + main.getRowFactor() - main.getColumnDelta() + 1, length2);
-            for (j = Math.max(0, i - main.getColumnDelta()); j < to; ++j) {
+            to = main.index2To(i + 1) - 1;
+            for (j = main.index2FromNo0(i + 1) - 1; j < to; ++j) {
                 match = main.get(i, j) +
                         scoring.getScore(seq1.codeAt(offset1 + i), seq2.codeAt(offset2 + j));
 
@@ -141,17 +158,17 @@ public class BandedAffineAligner {
         final int gapExtensionPenalty = scoring.getGapExtensionPenalty();
 
         for (i = 0; i < length1; ++i) {
-            to = Math.min(i + main.getRowFactor() - main.getColumnDelta() + 1, length2);
-            for (j = Math.max(0, i - main.getColumnDelta()); j < to; ++j) {
+            to = main.index2To(i + 1) - 1;
+            for (j = main.index2FromNo0(i + 1) - 1; j < to; ++j) {
                 match = main.get(i, j) +
                         scoring.getScore(seq1.codeAt(offset1 + i), seq2.codeAt(offset2 + j));
 
-                gap1 = Math.max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
-                gap2 = Math.max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
+                gap1 = max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
+                gap2 = max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
 
                 gapIn1.set(i + 1, j + 1, gap1);
                 gapIn2.set(i + 1, j + 1, gap2);
-                int score = Math.max(match, Math.max(gap1, gap2));
+                int score = max(match, max(gap1, gap2));
                 main.set(i + 1, j + 1, score);
 
                 if (score > maxScore) {
@@ -244,17 +261,17 @@ public class BandedAffineAligner {
         final int gapExtensionPenalty = scoring.getGapExtensionPenalty();
 
         for (i = 0; i < length1; ++i) {
-            to = Math.min(i + main.getRowFactor() - main.getColumnDelta() + 1, length2);
-            for (j = Math.max(0, i - main.getColumnDelta()); j < to; ++j) {
+            to = main.index2To(i + 1) - 1;
+            for (j = main.index2FromNo0(i + 1) - 1; j < to; ++j) {
                 match = main.get(i, j) +
                         scoring.getScore(seq1.codeAt(offset1 + length1 - 1 - i), seq2.codeAt(offset2 + length2 - 1 - j));
 
-                gap1 = Math.max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
-                gap2 = Math.max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
+                gap1 = max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
+                gap2 = max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
 
                 gapIn1.set(i + 1, j + 1, gap1);
                 gapIn2.set(i + 1, j + 1, gap2);
-                int score = Math.max(match, Math.max(gap1, gap2));
+                int score = max(match, max(gap1, gap2));
                 main.set(i + 1, j + 1, score);
 
                 if (score > maxScore) {
@@ -331,17 +348,17 @@ public class BandedAffineAligner {
         int gapExtensionPenalty = scoring.getGapExtensionPenalty();
 
         for (i = 0; i < length1; ++i) {
-            to = Math.min(i + main.getRowFactor() - main.getColumnDelta() + 1, length2);
-            for (j = Math.max(0, i - main.getColumnDelta()); j < to; ++j) {
+            to = main.index2To(i + 1) - 1;
+            for (j = main.index2FromNo0(i + 1) - 1; j < to; ++j) {
                 match = main.get(i, j) +
                         scoring.getScore(seq1.codeAt(offset1 + i), seq2.codeAt(offset2 + j));
 
-                gap1 = Math.max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
-                gap2 = Math.max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
+                gap1 = max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
+                gap2 = max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
 
                 gapIn1.set(i + 1, j + 1, gap1);
                 gapIn2.set(i + 1, j + 1, gap2);
-                main.set(i + 1, j + 1, Math.max(match, Math.max(gap1, gap2)));
+                main.set(i + 1, j + 1, max(match, max(gap1, gap2)));
             }
         }
 
@@ -432,17 +449,17 @@ public class BandedAffineAligner {
         int gapExtensionPenalty = scoring.getGapExtensionPenalty();
 
         for (i = 0; i < length1; ++i) {
-            to = Math.min(i + main.getRowFactor() - main.getColumnDelta() + 1, length2);
-            for (j = Math.max(0, i - main.getColumnDelta()); j < to; ++j) {
+            to = main.index2To(i + 1) - 1;
+            for (j = main.index2FromNo0(i + 1) - 1; j < to; ++j) {
                 match = main.get(i, j) +
                         scoring.getScore(seq1.codeAt(offset1 + length1 - 1 - i), seq2.codeAt(offset2 + length2 - 1 - j));
 
-                gap1 = Math.max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
-                gap2 = Math.max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
+                gap1 = max(main.get(i + 1, j) + scoring.getGapOpenPenalty(), gapIn1.get(i + 1, j) + gapExtensionPenalty);
+                gap2 = max(main.get(i, j + 1) + scoring.getGapOpenPenalty(), gapIn2.get(i, j + 1) + gapExtensionPenalty);
 
                 gapIn1.set(i + 1, j + 1, gap1);
                 gapIn2.set(i + 1, j + 1, gap2);
-                main.set(i + 1, j + 1, Math.max(match, Math.max(gap1, gap2)));
+                main.set(i + 1, j + 1, max(match, max(gap1, gap2)));
             }
         }
 
@@ -613,14 +630,14 @@ public class BandedAffineAligner {
             BandedMatrix gapIn1 = this.gapIn1 = new BandedMatrix(gapIn1Cache, size1, size2, width);
             BandedMatrix gapIn2 = this.gapIn2 = new BandedMatrix(gapIn2Cache, size1, size2, width);
 
-            for (int i = main.getRowFactor() - main.getColumnDelta(); i > 0; --i) {
-                int v = scoring.getGapOpenPenalty() + scoring.getGapExtensionPenalty() * (i - 1);
-                main.set(0, i, v);
-                gapIn1.set(0, i, v);
-                gapIn2.set(0, i, BandedMatrix.DEFAULT_VALUE);
+            for (int j = main.index2To(0) - 1; j > 0; --j) {
+                int v = scoring.getGapOpenPenalty() + scoring.getGapExtensionPenalty() * (j - 1);
+                main.set(0, j, v);
+                gapIn1.set(0, j, v);
+                gapIn2.set(0, j, BandedMatrix.DEFAULT_VALUE);
             }
 
-            for (int i = main.getColumnDelta(); i > 0; --i) {
+            for (int i = main.index1To(0) - 1; i > 0; --i) {
                 int v = scoring.getGapOpenPenalty() + scoring.getGapExtensionPenalty() * (i - 1);
                 main.set(i, 0, v);
                 gapIn1.set(i, 0, BandedMatrix.DEFAULT_VALUE);
