@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,7 +16,7 @@
 package com.milaboratory.primitivio;
 
 public interface Serializer<T> {
-    void write(PrimitivO output, T object);
+    void write(PrimitivO output, T obj);
 
     T read(PrimitivI input);
 
