@@ -1,3 +1,18 @@
+/*
+ * Copyright (c) 2022 MiLaboratories Inc.
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 package com.milaboratory.core.alignment;
 
 import com.milaboratory.core.Range;
@@ -15,7 +30,7 @@ import org.junit.Test;
 
 import java.util.Arrays;
 
-import static com.milaboratory.core.alignment.AlignerTest.assertAlignment;
+import static com.milaboratory.core.alignment.AlignmentTestUtils.assertAlignment;
 import static com.milaboratory.core.alignment.BandedAffineAligner.semiLocalLeft;
 import static com.milaboratory.test.TestUtil.its;
 import static com.milaboratory.test.TestUtil.randomSequence;
@@ -382,11 +397,11 @@ public class BandedAffineAlignerTest {
                 Assert.assertEquals(r.getRelativeMutations().mutate(sequence.getRange(r.getSequence1Range())),
                         mutated.getRange(r.getSequence2Range()));
 
-                AlignerTest.assertAlignment(r, mutated, sc);
+                AlignmentTestUtils.assertAlignment(r, mutated, sc);
                 Assert.assertTrue(mutScore <= r.calculateScore(sc));
 
                 r = Aligner.alignGlobal(sc, mutated, sequence);
-                AlignerTest.assertAlignment(r, sequence, sc);
+                AlignmentTestUtils.assertAlignment(r, sequence, sc);
 
                 Assert.assertEquals(r.getRelativeMutations().mutate(mutated.getRange(r.getSequence1Range())),
                         sequence.getRange(r.getSequence2Range()));
@@ -396,4 +411,4 @@ public class BandedAffineAlignerTest {
             }
         }
     }
-}
\ No newline at end of file
+}
