@@ -1,5 +1,5 @@
 /*
- * Copyright 2015 MiLaboratory.com
+ * Copyright (c) 2022 MiLaboratories Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -28,8 +28,7 @@ import org.apache.commons.math3.random.Well19937c;
 import org.junit.Assert;
 import org.junit.Test;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.*;
 
 public class BitapPatternTest {
     @Test
@@ -107,7 +106,7 @@ public class BitapPatternTest {
         System.out.println(alignment.convertToSeq2Position(5));
 
         int pos;
-        while ((pos = bitapMatcher.findNext()) > 0){
+        while ((pos = bitapMatcher.findNext()) > 0) {
             System.out.println(pos);
             System.out.println(bitapMatcher.getNumberOfErrors());
             System.out.println();
@@ -225,7 +224,7 @@ public class BitapPatternTest {
 
         out:
         for (int i = 0; i < its; ++i) {
-            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 5, 60);
+            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 5, 60);
 
             NucleotideSequence seqM = seq;
             int mms = 1 + rg.nextInt(Math.min(10, seq.size()));
@@ -234,8 +233,8 @@ public class BitapPatternTest {
 
             int realMMs = SequencesUtils.mismatchCount(seq, 0, seqM, 0, seqM.size());
 
-            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
-            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
+            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
+            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
             NucleotideSequence fullSeq = SequencesUtils.concatenate(seqLeft, seqM, seqRight);
 
             Motif<NucleotideSequence> motif = new Motif<>(seq);
@@ -271,7 +270,7 @@ public class BitapPatternTest {
 
         out:
         for (int i = 0; i < its; ++i) {
-            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 10, 60);
+            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 10, 60);
 
             NucleotideSequence seqM = seq;
             int mms = 1 + rg.nextInt(3);
@@ -280,8 +279,8 @@ public class BitapPatternTest {
 
             int realMMs = SequencesUtils.mismatchCount(seq, 0, seqM, 0, seqM.size());
 
-            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
-            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
+            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
+            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
             NucleotideSequence fullSeq = SequencesUtils.concatenate(seqLeft, seqM, seqRight);
 
             Motif<NucleotideSequence> motif = new Motif<>(seq);
@@ -318,15 +317,15 @@ public class BitapPatternTest {
 
         out:
         for (int i = 0; i < its; ++i) {
-            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 5, 60);
+            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 5, 60);
 
             NucleotideSequence seqM = seq;
             int muts = 1 + rg.nextInt(Math.min(10, seq.size()));
             for (int j = 0; j < muts; ++j)
                 seqM = UniformMutationsGenerator.createUniformMutationAsObject(seqM, rg).mutate(seqM);
 
-            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
-            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
+            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
+            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
             NucleotideSequence fullSeq = SequencesUtils.concatenate(seqLeft, seqM, seqRight);
 
             Motif<NucleotideSequence> motif = new Motif<>(seq);
@@ -354,15 +353,15 @@ public class BitapPatternTest {
 
         out:
         for (int i = 0; i < its; ++i) {
-            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 5, 60);
+            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 5, 60);
 
             NucleotideSequence seqM = seq;
             int muts = 1 + rg.nextInt(Math.min(10, seq.size()));
             for (int j = 0; j < muts; ++j)
                 seqM = UniformMutationsGenerator.createUniformMutationAsObject(seqM, rg).mutate(seqM);
 
-            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
-            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, 0, 40);
+            NucleotideSequence seqLeft = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
+            NucleotideSequence seqRight = TestUtil.randomSequence(NucleotideSequence.ALPHABET, rg, 0, 40);
             NucleotideSequence fullSeq = SequencesUtils.concatenate(seqLeft, seqM, seqRight);
 
             Motif<NucleotideSequence> motif = new Motif<>(seq);
@@ -380,4 +379,142 @@ public class BitapPatternTest {
             assertTrue("On iteration = " + i + " with seed " + seed, found);
         }
     }
-}
\ No newline at end of file
+
+    @Test
+    public void scoringTest1() {
+        Motif<NucleotideSequence> motif = new NucleotideSequence("ATTAGACA").toMotif();
+        BitapPattern bitap = motif.getBitapPattern();
+
+        BitapMatcherWithScore matcher = bitap.exactMatcher(new NucleotideSequence("ATTAGACA"), 0, 8);
+        assertEquals(0, matcher.findNext());
+        assertEquals(motif.matchBitScore(), matcher.getBitScore(), 0.1);
+        assertFalse(Double.isNaN(matcher.getBitScoreCost()));
+
+        matcher = bitap.exactMatcher(new NucleotideSequence("ATTAGACN"), 0, 8);
+        assertEquals(0, matcher.findNext());
+        assertEquals(motif.matchBitScore() - 2, matcher.getBitScore(), 0.1);
+        assertFalse(Double.isNaN(matcher.getBitScoreCost()));
+
+        motif = new NucleotideSequence("ATTAGACN").toMotif();
+        bitap = motif.getBitapPattern();
+        matcher = bitap.exactMatcher(new NucleotideSequence("ATTAGANT"), 0, 8);
+        assertEquals(0, matcher.findNext());
+        assertEquals(motif.matchBitScore() - 2, matcher.getBitScore(), 0.1);
+        assertFalse(Double.isNaN(matcher.getBitScoreCost()));
+    }
+
+    @Test
+    public void scoringTest2() {
+        Motif<NucleotideSequence> motif = new NucleotideSequence("ATTAGACA").toMotif();
+        BitapPattern bitap = motif.getBitapPattern();
+
+        long n = 0;
+        int K = 1000000;
+        int S = 100;
+        for (int i = 0; i < K; i++) {
+            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET,
+                    S + motif.size(), S + motif.size());
+            BitapMatcherWithScore matcher = bitap.substitutionOnlyMatcherFirst(1, seq);
+            if (matcher.findNext() > 0)
+                ++n;
+        }
+
+        BitapMatcherWithScore matcher = bitap.substitutionOnlyMatcherFirst(1, new NucleotideSequence("TTTAGACA"));
+        assertEquals(0, matcher.findNext());
+
+        assertEquals(-Math.log(1.0 * n / K / S) / Math.log(2), matcher.getBitScore(), 1.0);
+        assertFalse(Double.isNaN(matcher.getBitScoreCost()));
+    }
+
+    @Test
+    public void scoringTest3() {
+        Motif<NucleotideSequence> motif = new NucleotideSequence("ATTAGACA").toMotif();
+        BitapPattern bitap = motif.getBitapPattern();
+
+        long n = 0;
+        int K = 1000000;
+        int S = 100;
+        for (int i = 0; i < K; i++) {
+            NucleotideSequence seq = TestUtil.randomSequence(NucleotideSequence.ALPHABET,
+                    S + motif.size(), S + motif.size());
+            BitapMatcherWithScore matcher = bitap.substitutionOnlyMatcherFirst(2, seq);
+            if (matcher.findNext() > 0)
+                ++n;
+        }
+
+        BitapMatcherWithScore matcher = new BitapMatcherFilter(bitap.substitutionOnlyMatcherFirst(2, new NucleotideSequence("TTAAGACA")));
+        assertEquals(0, matcher.findNext());
+
+        assertEquals(-Math.log(1.0 * n / K / S) / Math.log(2), matcher.getBitScore(), 1.0);
+        assertFalse(Double.isNaN(matcher.getBitScoreCost()));
+    }
+
+    @Test
+    public void testMismatch1ExactMask() {
+        BitapPattern pattern = MotifWithExactMask.fromNucleotide("attAGAcag").toBitapPattern();
+
+        NucleotideSequence seq;
+        BitapMatcherWithScore bitapMatcher;
+
+        // Exact
+        seq = new NucleotideSequence("ACTGCGATAAATTAGACAGTACGTA");
+        bitapMatcher = new BitapMatcherFilter(pattern.substitutionOnlyMatcherFirst(1, seq));
+        boolean t = false;
+        int pos;
+        while ((pos = bitapMatcher.findNext()) > 0)
+            if (bitapMatcher.getNumberOfErrors() == 0) {
+                t = true;
+                break;
+            }
+        assertTrue(t);
+        assertEquals(10, pos);
+
+        // Mismatch
+        //                            ACTGCGATAAATTAgACAGTACGTA
+        seq = new NucleotideSequence("ACTGCGATAAATTAcACAGTACGTA");
+        bitapMatcher = new BitapMatcherFilter(pattern.substitutionOnlyMatcherFirst(1, seq));
+        assertEquals(-1, bitapMatcher.findNext());
+
+        // Mismatch
+        //                            ACTGCGATAAATTAGAcAGTACGTA
+        seq = new NucleotideSequence("ACTGCGATAAATTAGAgAGTACGTA");
+        bitapMatcher = new BitapMatcherFilter(pattern.substitutionOnlyMatcherFirst(1, seq));
+        assertEquals(10, bitapMatcher.findNext());
+        assertEquals(1, bitapMatcher.getNumberOfErrors());
+        assertEquals(-1, bitapMatcher.findNext());
+    }
+
+    @Test
+    public void testMismatch2ExactMask() {
+        BitapPattern pattern = MotifWithExactMask.fromNucleotide("attAGAcag").toBitapPattern();
+
+        NucleotideSequence seq;
+        BitapMatcherWithScore bitapMatcher;
+
+        // Exact
+        seq = new NucleotideSequence("ACTGCGATAAATTAGACAGTACGTA");
+        bitapMatcher = pattern.substitutionOnlyMatcherFirst(1, seq);
+        boolean t = false;
+        int pos;
+        while ((pos = bitapMatcher.findNext()) > 0)
+            if (bitapMatcher.getNumberOfErrors() == 0) {
+                t = true;
+                break;
+            }
+        assertTrue(t);
+        assertEquals(10, pos);
+
+        double exactMatchBitScore = bitapMatcher.getBitScore();
+
+        // Mismatch
+        //                            ACTGCGATAAATTAGAcAGTACGTA
+        seq = new NucleotideSequence("ACTGCGATAAATTAGAgAGTACGTA");
+        bitapMatcher = new BitapMatcherFilter(pattern.substitutionOnlyMatcherFirst(1, seq));
+        assertEquals(10, bitapMatcher.findNext());
+        double oneMismatchMatchBitScore = bitapMatcher.getBitScore();
+        assertEquals(1, bitapMatcher.getNumberOfErrors());
+        assertEquals(-1, bitapMatcher.findNext());
+
+        assertEquals(pattern.getAverageMismatchPenalty(), exactMatchBitScore - oneMismatchMatchBitScore, 0.2);
+    }
+}
